package net.javase.hksup.starter.service;

import com.sun.jna.Native;
import lombok.extern.slf4j.Slf4j;
import net.javase.hksup.starter.config.HkSupSdkProperties;
import net.javase.hksup.starter.sdk.HCISUPCMS;
import net.javase.hksup.starter.sdk.HCISUPSS;
import net.javase.hksup.starter.utils.SystemUtil;
import org.springframework.context.ApplicationContext;

import java.util.Map;
import java.util.Optional;

/**
 * SsServer
 *
 * @author Frank
 */
@Slf4j
public class SsServer {

    public static HCISUPSS hCEhomeSS = null;
    String PSS_CLIENT_FILE_PATH_PARAM_NAME = "File-Path"; //图片文件路径
    String PSS_CLIENT_VRB_FILENAME_CODE = "Filename-Code";//VRB协议的FilenameCode
    String PSS_CLIENT_KMS_USER_NAME = "KMS-Username"; //KMS图片服务器用户名
    String PSS_CLIENT_KMS_PASSWORD = "KMS-Password"; //KMS图片服务器密码
    String PSS_CLIENT_CLOUD_AK_NAME = "Access-Key"; //云存储协议AcessKey
    String PSS_CLIENT_CLOUD_SK_NAME = "Secret-Key"; //云存储协议SecretKey

    private HCISUPSS.EHomeSSMsgCallBack eHomeSSMsgCallBack;// 信息回调函数(上报)
    private HCISUPSS.EHomeSSStorageCallBack eHomeSSStorageCallBack;// 文件保存回调函数(下载)
    private Optional<HCISUPSS.EHomeSSRWCallBackEx> eHomeSSRWCallBackEx; //读写扩展回调函数
    private HCISUPSS.NET_EHOME_SS_LISTEN_PARAM pSSListenParam = new HCISUPSS.NET_EHOME_SS_LISTEN_PARAM();
    private int ssHandle = -1; //存储服务监听句柄
    int client = -1;
    private static int iCount = 0;
    byte[] szUrl = new byte[HCISUPSS.MAX_URL_LEN_SS];
    String url = "";

    private final HkSupSdkProperties hkSupSdkProperties;

    private final ApplicationContext applicationContext;

    public SsServer(HkSupSdkProperties hkSupSdkProperties, ApplicationContext applicationContext) {
        this.hkSupSdkProperties = hkSupSdkProperties;
        this.applicationContext = applicationContext;
        createSDKInstance();
    }

    /**
     * 根据不同操作系统选择不同的库文件和库路径
     */
    private boolean createSDKInstance() {
        String libPath = hkSupSdkProperties.libPath();
        if (hCEhomeSS == null) {
            synchronized (HCISUPSS.class) {
                String strDllPath = "";
                try {
                    SystemUtil.jnaDebugLoad(hkSupSdkProperties.getJnaDebug());
                    if (SystemUtil.isWindows()) {
                        strDllPath = libPath + "HCISUPSS.dll";
                    } else if (SystemUtil.isLinux()) {
                        strDllPath = libPath + "libHCISUPSS.so";
                    }
                    log.info("loadLibrary: {}", strDllPath);
                    hCEhomeSS = (HCISUPSS) Native.loadLibrary(strDllPath, HCISUPSS.class);
                } catch (Exception ex) {
                    log.error("loadLibrary: {}, Error: {}", strDllPath, ex.getMessage());
                    return false;
                }
            }
        }
        return true;
    }

    private void eSSInit() {
        if (hCEhomeSS == null) {
            if (!createSDKInstance()) {
                throw new RuntimeException("Load SS SDK fail");
            }
        }
        String libPath = hkSupSdkProperties.libPath();
        if (SystemUtil.isWindows()) {
            String strPathCrypto = libPath + "libeay32.dll"; //Linux版本是libcrypto.so库文件的路径
            int iPathCryptoLen = strPathCrypto.getBytes().length;
            HCISUPCMS.BYTE_ARRAY ptrByteArrayCrypto = new HCISUPCMS.BYTE_ARRAY(iPathCryptoLen + 1);
            System.arraycopy(strPathCrypto.getBytes(), 0, ptrByteArrayCrypto.byValue, 0, iPathCryptoLen);
            ptrByteArrayCrypto.write();
            log.info(new String(ptrByteArrayCrypto.byValue));
            hCEhomeSS.NET_ESS_SetSDKInitCfg(4, ptrByteArrayCrypto.getPointer());

            //设置libssl.so所在路径
            String strPathSsl = libPath + "ssleay32.dll";    //Linux版本是libssl.so库文件的路径
            int iPathSslLen = strPathSsl.getBytes().length;
            HCISUPCMS.BYTE_ARRAY ptrByteArraySsl = new HCISUPCMS.BYTE_ARRAY(iPathSslLen + 1);
            System.arraycopy(strPathSsl.getBytes(), 0, ptrByteArraySsl.byValue, 0, iPathSslLen);
            ptrByteArraySsl.write();
            log.info(new String(ptrByteArraySsl.byValue));
            hCEhomeSS.NET_ESS_SetSDKInitCfg(5, ptrByteArraySsl.getPointer());

            //设置sqlite3库的路径
            String strPathSqlite = libPath + "sqlite3.dll";
            int iPathSqliteLen = strPathSqlite.getBytes().length;
            HCISUPCMS.BYTE_ARRAY ptrByteArraySqlite = new HCISUPCMS.BYTE_ARRAY(iPathSqliteLen + 1);
            System.arraycopy(strPathSqlite.getBytes(), 0, ptrByteArraySqlite.byValue, 0, iPathSqliteLen);
            ptrByteArraySqlite.write();
            log.info(new String(ptrByteArraySqlite.byValue));
            hCEhomeSS.NET_ESS_SetSDKInitCfg(6, ptrByteArraySqlite.getPointer());
            //SDK初始化
            boolean sinit = hCEhomeSS.NET_ESS_Init();
            if (!sinit) {
                log.info("NET_ESS_Init失败，错误码：{}", hCEhomeSS.NET_ESS_GetLastError());
            }
            //设置图片存储服务器公网地址 （当存在内外网映射时使用
            HCISUPCMS.NET_EHOME_IPADDRESS ipaddress = new HCISUPCMS.NET_EHOME_IPADDRESS();
            String ServerIP = hkSupSdkProperties.getPicServerIp();
            System.arraycopy(ServerIP.getBytes(), 0, ipaddress.szIP, 0, ServerIP.length());
            ipaddress.wPort = Short.parseShort(hkSupSdkProperties.getPicServerPort().toString());
            ipaddress.write();
            boolean b = hCEhomeSS.NET_ESS_SetSDKInitCfg(3, ipaddress.getPointer());
            if (!b) {
                log.info("NET_ESS_SetSDKInitCfg失败，错误码：{}", hCEhomeSS.NET_ESS_GetLastError());
            }

        } else if (SystemUtil.isLinux()) {
            HCISUPCMS.BYTE_ARRAY ptrByteArrayCrypto = new HCISUPCMS.BYTE_ARRAY(256);
            String strPathCrypto = libPath + "libcrypto.so"; //Linux版本是libcrypto.so库文件的路径
            log.info(strPathCrypto);
            System.arraycopy(strPathCrypto.getBytes(), 0, ptrByteArrayCrypto.byValue, 0, strPathCrypto.length());
            ptrByteArrayCrypto.write();
            hCEhomeSS.NET_ESS_SetSDKInitCfg(4, ptrByteArrayCrypto.getPointer());

            //设置libssl.so所在路径
            HCISUPCMS.BYTE_ARRAY ptrByteArraySsl = new HCISUPCMS.BYTE_ARRAY(256);
            String strPathSsl = libPath + "libssl.so";    //Linux版本是libssl.so库文件的路径
            log.info(strPathSsl);
            System.arraycopy(strPathSsl.getBytes(), 0, ptrByteArraySsl.byValue, 0, strPathSsl.length());
            ptrByteArraySsl.write();
            hCEhomeSS.NET_ESS_SetSDKInitCfg(5, ptrByteArraySsl.getPointer());

            //设置splite3.so所在路径
            HCISUPCMS.BYTE_ARRAY ptrByteArraysplite = new HCISUPCMS.BYTE_ARRAY(256);
            String strPathsplite = libPath + "libsqlite3.so";    //Linux版本是libsqlite3.so库文件的路径
            log.info(strPathsplite);
            System.arraycopy(strPathsplite.getBytes(), 0, ptrByteArraysplite.byValue, 0, strPathsplite.length());
            ptrByteArraysplite.write();
            hCEhomeSS.NET_ESS_SetSDKInitCfg(6, ptrByteArraysplite.getPointer());
            //SDK初始化
            boolean sinit = hCEhomeSS.NET_ESS_Init();
            if (!sinit) {
                log.info("NET_ESS_Init失败，错误码：{}", hCEhomeSS.NET_ESS_GetLastError());
            }
            //设置图片存储服务器公网地址 （当存在内外网映射时使用
            HCISUPCMS.NET_EHOME_IPADDRESS ipaddress = new HCISUPCMS.NET_EHOME_IPADDRESS();
            String ServerIP = hkSupSdkProperties.getPicServerIp();
            System.arraycopy(ServerIP.getBytes(), 0, ipaddress.szIP, 0, ServerIP.length());
            ipaddress.wPort = Short.parseShort(hkSupSdkProperties.getPicServerPort().toString());
            ipaddress.write();
            boolean b = hCEhomeSS.NET_ESS_SetSDKInitCfg(3, ipaddress.getPointer());
            if (!b) {
                log.info("NET_ESS_SetSDKInitCfg失败，错误码：{}", hCEhomeSS.NET_ESS_GetLastError());
            }

        }
        //启用SDK写日志
        boolean logToFile = hCEhomeSS.NET_ESS_SetLogToFile(3, hkSupSdkProperties.logPath(), false);
    }

    /**
     * 开启存储服务监听
     */
    private void startSsListen() {
        String SSIP = hkSupSdkProperties.getPicServerListenIp();
        System.arraycopy(SSIP.getBytes(), 0, pSSListenParam.struAddress.szIP, 0, SSIP.length());
        pSSListenParam.struAddress.wPort = Short.parseShort(hkSupSdkProperties.getPicServerListenPort().toString());
        String strKMS_UserName = hkSupSdkProperties.getKmsUsername();
        System.arraycopy(strKMS_UserName.getBytes(), 0, pSSListenParam.szKMS_UserName, 0, strKMS_UserName.length());
        String strKMS_Password = hkSupSdkProperties.getKmsPassword();
        System.arraycopy(strKMS_Password.getBytes(), 0, pSSListenParam.szKMS_Password, 0, strKMS_Password.length());
        String strAccessKey = hkSupSdkProperties.getSzAk();
        System.arraycopy(strAccessKey.getBytes(), 0, pSSListenParam.szAccessKey, 0, strAccessKey.length());
        String strSecretKey = hkSupSdkProperties.getSzSk();
        System.arraycopy(strSecretKey.getBytes(), 0, pSSListenParam.szSecretKey, 0, strSecretKey.length());
        pSSListenParam.byHttps = 0;
        /******************************************************************
         * 存储信息回调
         */
        if (eHomeSSMsgCallBack == null) {
            Map<String, HCISUPSS.EHomeSSMsgCallBack> beansOfType = applicationContext.getBeansOfType(HCISUPSS.EHomeSSMsgCallBack.class);
            if (beansOfType.size() < 1) {
                throw new RuntimeException("not found EHomeSSMsgCallBack implement!");
            }
            eHomeSSMsgCallBack = beansOfType.values().stream().findFirst().get();
        }
        pSSListenParam.fnSSMsgCb = eHomeSSMsgCallBack;

        /******************************************************************
         * 存储数据回调
         * fnSStorageCb或者fnSSRWCbEx，只需要设置一种回调函数
         * 简单功能测试可以使用存储回调(SDK底层使用db数据库自动存取数据，因此会受到db数据库的性能瓶颈影响)
         * 需要自定义URL或者自己读写图片数据，则使用读写扩展回调(推荐)
         */
        //读写扩展回调
        eHomeSSRWCallBackEx = applicationContext.getBeansOfType(HCISUPSS.EHomeSSRWCallBackEx.class)
                .values().stream().findFirst();

        if (eHomeSSRWCallBackEx.isPresent()) {
            pSSListenParam.fnSSRWCbEx = eHomeSSRWCallBackEx.get();
        } else {
            //存储信息回调
            if (eHomeSSStorageCallBack == null) {
                Map<String, HCISUPSS.EHomeSSStorageCallBack> beansOfType = applicationContext.getBeansOfType(HCISUPSS.EHomeSSStorageCallBack.class);
                if (beansOfType.size() < 1) {
                    throw new RuntimeException("not found eHomeSSStorageCallBack implement!");
                }
                eHomeSSStorageCallBack = beansOfType.values().stream().findFirst().get();
            }
            pSSListenParam.fnSStorageCb = eHomeSSStorageCallBack;
        }

        pSSListenParam.bySecurityMode = 1;
        pSSListenParam.write();
        ssHandle = hCEhomeSS.NET_ESS_StartListen(pSSListenParam);
        if (ssHandle == -1) {
            int err = hCEhomeSS.NET_ESS_GetLastError();
            log.info("NET_ESS_StartListen failed,error: {}", err);
            hCEhomeSS.NET_ESS_Fini();
        } else {
            String SsListenInfo = new String(pSSListenParam.struAddress.szIP).trim() + "_" + pSSListenParam.struAddress.wPort;
            log.info("存储服务器：{}, NET_ESS_StartListen succeed!\n", SsListenInfo);
        }
    }

    /**
     * 创建存储客户端
     */
    private void ssCreateClient(String filePath) {
        HCISUPSS.NET_EHOME_SS_CLIENT_PARAM pClientParam = new HCISUPSS.NET_EHOME_SS_CLIENT_PARAM();
        //存储服务器类型
        switch (hkSupSdkProperties.getPicServerType()) {
            case 1:
                pClientParam.enumType = HCISUPSS.NET_EHOME_SS_CLIENT_TYPE_VRB;
                break;
            case 2:
                pClientParam.enumType = HCISUPSS.NET_EHOME_SS_CLIENT_TYPE_CLOUD;
                break;
            case 3:
                pClientParam.enumType = HCISUPSS.NET_EHOME_SS_CLIENT_TYPE_KMS;
                break;
            default:
                log.info("存储服务器类型选择错误");
                break;
        }
        pClientParam.struAddress.szIP = hkSupSdkProperties.getPicServerIp().getBytes();
        pClientParam.struAddress.wPort = Short.parseShort(hkSupSdkProperties.getPicServerPort().toString());
        pClientParam.byHttps = 0;
        pClientParam.write();

        client = hCEhomeSS.NET_ESS_CreateClient(pClientParam);
        if (client < 0) {
            int err = hCEhomeSS.NET_ESS_GetLastError();
            log.warn("创建图片上传/下载客户端出错,错误号：{}, client={}", err, client);
        } else {
            if (!hCEhomeSS.NET_ESS_ClientSetTimeout(client, 6000, 6000)) {
                int err = hCEhomeSS.NET_ESS_GetLastError();
                log.warn("NET_ESS_ClientSetTimeout失败,错误号：{}", err);
            }
            boolean bSetParam = hCEhomeSS.NET_ESS_ClientSetParam(client, PSS_CLIENT_FILE_PATH_PARAM_NAME, filePath);
            boolean bKMS_UserName = hCEhomeSS.NET_ESS_ClientSetParam(client, "SS_CLIENT_KMS_USER_NAME", hkSupSdkProperties.getKmsUsername());
            if (!bKMS_UserName) {
                log.warn("NET_ESS_ClientSetParam失败\n err: {}", hCEhomeSS.NET_ESS_GetLastError());
            }
            boolean bKMS_PassWord = hCEhomeSS.NET_ESS_ClientSetParam(client, "SS_CLIENT_KMS_PASSWIRD", hkSupSdkProperties.getKmsPassword());
            if (!bKMS_PassWord) {
                log.warn("NET_ESS_ClientSetParam失败\n err: {}", hCEhomeSS.NET_ESS_GetLastError());
            }
            boolean bCloud_AK = hCEhomeSS.NET_ESS_ClientSetParam(client, PSS_CLIENT_CLOUD_AK_NAME, hkSupSdkProperties.getSzAk());
            boolean bCloud_SK = hCEhomeSS.NET_ESS_ClientSetParam(client, PSS_CLIENT_CLOUD_SK_NAME, hkSupSdkProperties.getSzSk());
            log.info("ssCreateClient: bSetParam: {},bKMS_UserName: {}, bKMS_PassWord: {}, bCloud_AK: {}, bCloud_SK: {}",
                    bSetParam, bKMS_UserName, bKMS_PassWord, bCloud_AK, bCloud_SK);
        }
    }

    /**
     * 上传图片到存储服务，返回URL
     */
    private String ssUploadPic() {
        iCount++;
        szUrl = new byte[HCISUPSS.MAX_URL_LEN_SS];

        boolean doUpload = hCEhomeSS.NET_ESS_ClientDoUpload(client, szUrl, HCISUPSS.MAX_URL_LEN_SS - 1);
        if (!doUpload) {
            int err = hCEhomeSS.NET_ESS_GetLastError();
            log.warn("NET_ESS_ClientDoUpload失败，错误号：{}", err);
            return null;
        } else {
            url = "http://" + hkSupSdkProperties.getPicServerIp() + ":" + hkSupSdkProperties.getPicServerPort() + new String(szUrl).trim();
            log.info("NET_ESS_ClientDoUpload succeed, Count: {}, Pic Url: {}", iCount, url);
            return url;
        }
    }

    /**
     * 销毁存储客户端
     */
    private void ssDestroyClient() {
        if (hCEhomeSS.NET_ESS_DestroyClient(client))//释放资源
        {
            client = -1;
        }
    }

    /**
     * 上传图片返回url
     *
     * @param fileAbsPath 文件路径
     * @return url
     */
    public String uploadPic(String fileAbsPath) {
        ssCreateClient(fileAbsPath);
        String url = ssUploadPic();
        ssDestroyClient();
        return url;
    }

    /**
     * 启动服务
     */
    void startSsServer() {
        try {
            eSSInit();
            startSsListen();
        } catch (Exception ex) {
            log.error("启动SS存储服务失败: " + ex.getMessage(), ex);
        }
    }

    /**
     * 停止服务, 销毁
     */
    void stopSsServer() {
        if (ssHandle >= 0) {
            try {
                log.info("开始停止存储SS服务");
                hCEhomeSS.NET_ESS_StopListen(ssHandle);
                hCEhomeSS.NET_ESS_Fini();
                log.info("停止存储SS服务成功!");
            } catch (Exception ex) {
                log.error("停止存储SS服务失败: " + ex.getMessage(), ex);
            }
        }
    }

}
